/* ESP32_sMQTTBroker_BME280
 *  
 * sMQTTBroker demo for ESP32 supplemented with data publishing in JSON format
 * 
 * A program egyedi MQTT brókert definiál, a virtuális visszakívási 
 * függvények felüldefiniálásával.
 * Az onConnect függvényt jelszóellenőrzéssel egészítettük ki.
 * Mindegyik visszahívási függvényt naplózással egészítettük ki.
 * 
 * A bróker funkciók ellátásán kívül 5 másodpercenként publikáljuk az I2C buszra 
 * kötött BME280 szernzor által mért adatokat a cspista/home/bme280 témakörbe,
 * JSON formátumban: {"temperature":"xx.x","humidity":"xx","pressure":"xxxx.x"}
 * Kapcsolódjunk egy külső klienssel, és publikáljunk valamit!
 * A soros porthoz kapcsolódó terminál ablakban követhetjük az eseményeket.
 * 
 * Forrás: https://github.com/terrorsl/sMQTTBroker
 * A felhasznált mintapélda: examples/advanvebroker.ino
 * Szerző: Vyacheslav Shiryaev
 * Kisebb módosítások: I. Cserny, Március 9., 2022. 
 * 
 * Tested with Arduino IDE 1.8.19 and ESP32 Arduino core 2.0.2
 */


#include <WiFi.h>
#include <sMQTTBroker.h>
#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>
#include <ArduinoJson.h>
#include "secrets.h"


Adafruit_BME280 bme;          // I2C
StaticJsonDocument<200> doc;

unsigned long lastMsg;

class MyBroker: public sMQTTBroker {
  public:
    bool onConnect(sMQTTClient *client, const std::string &username, const std::string &password)
  { if ( (0 == username.compare(uMQTT_USER)) && (0 == password.compare(uMQTT_PASS))) {
        /* if ((0 == strcmp(username.c_str(), uMQTT_USER)) && (0 == strcmp(password.c_str(), uMQTT_PASS))) { */
        std::printf("Client %s connected\r\n", client->getClientId().c_str());
        return true;
      }
      else {
        std::printf("Client %s failed to connect\r\n", client->getClientId().c_str());
        return false;
      }
    };
    void onRemove(sMQTTClient* client)
    {
      std::printf("Client %s disconnected\r\n", client->getClientId().c_str());

    };
    void onPublish(sMQTTClient *client, const std::string &topic, const std::string &payload)
    {
      // client publish a message to the topic
      // you can parse the message, save it to a database, or whatever, based on your goals
      std::printf("Client %s published %s into %s\r\n", client->getClientId().c_str(), payload.c_str(), topic.c_str());
    }
    bool onEvent(sMQTTEvent *event)
    {
      switch (event->Type())
      {
        case NewClient_sMQTTEventType:
          {
            sMQTTNewClientEvent *e = (sMQTTNewClientEvent*)event;
            e->Login();
            e->Password();
          }
          break;
        case LostConnect_sMQTTEventType:
          WiFi.reconnect();
          break;
      }
      return true;
    }
};

MyBroker broker;

void setup_wifi() {
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(WIFI_SSID);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("WiFi connected");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
}

void setup() {
  Serial.begin(115200);
  unsigned status = bme.begin(0x76, &Wire);
  if (!status) {
    Serial.println("Could not find a valid BME280 sensor, check wiring, address, sensor ID!");
    Serial.print("SensorID was: 0x"); Serial.println(bme.sensorID(), 16);
    Serial.print("        ID of 0xFF probably means a bad address, a BMP 180 or BMP 085\n");
    Serial.print("   ID of 0x56-0x58 represents a BMP 280,\n");
    Serial.print("        ID of 0x60 represents a BME 280.\n");
    Serial.print("        ID of 0x61 represents a BME 680.\n");
    while (1) delay(10);
  }

  setup_wifi();
  broker.init(1883);
};

void loop() {
  broker.update();

  if ((millis() - lastMsg) > 5000) {
    //--- Read and publish BME280 sensor data ---------
    doc["temperature"] = String(bme.readTemperature(), 1);
    doc["humidity"] = (int)bme.readHumidity();
    doc["pressure"] = String((bme.readPressure()/98.58566F), 1); // with sea level correction for Debrecen
    std::string payload = doc.as<std::string>();
    std::string mytopic = "cspista/home/bme280";
    broker.publish(mytopic, payload, 0, false);
    lastMsg += 5000;
  }

};
