/* ESP32_sMQTT_DHT22_neopixel
 *  
 * MQTT kliens program, amely egy DHT22 szenzor adatait publikálja 
 * a helyi hálózaton a cspista/home/temp és cspista/home/humidity
 * témakörökbe. A szenzor a GPIO23 kivezetésre csatlakozik.
 * A kliens ugyanakkor feliratkozik a cspista/home/led témakörre és
 * a beérkező üzenetek első karakterével (0 vagy 1) a beépített LED-et
 * be- vagy kikapcsoljuk.
 * A kliens feliratkozik a cspista/home/rgb témakörre is és a beérkező
 * #RRGGBB formátumú hexadecimális üzenetekkel egy WS2812 (Neopixel)
 * LEDET vezérelünk, ami az ESP32 kártya GPIO22 kivezetésére
 * csatlakozik.
 * 
 * A 192.168.1.30 fix IP számon elérhető lokális MQTT brókerhez 
 * itt a secrets.h állományban uMQTT_USER és uMQTT_PASS névvel
 * definiált autentikációs adatokkal jeletkezünk be, de a 
 * program fel van készítve a broker.hivemq.com MQTT brokerhez
 * történő kapcsolódásra is a HIVEMQ_USER és HIVEMQ_PASS névvel 
 * definiált autentikációs adatokkal, csupán két sort kell kicserélni.
 * 
 * Tested with Arduino IDE 1.8.19 and ESP32 Arduino core 2.0.2
 */
#include <WiFi.h>
#include <PubSubClient.h>
#include <Adafruit_NeoPixel.h>
#include "DHT.h"
#include "secrets.h"
// const char* mqtt_server = "broker.hivemq.com"; // remote server: HiveMQ broker
const char* mqtt_server = "192.168.1.30";         // local server: ESP32 sMQTTBroker
#define DHTPIN          23     // GPIO23 pin for DHT sensor
#define NEOPIXEL_PIN    22     // GPIO22 (D2) pin for Neopixel 
#define NUMPIXELS        1     // NeoPixel string size 
Adafruit_NeoPixel pixels(NUMPIXELS, NEOPIXEL_PIN, NEO_GRB + NEO_KHZ800);
DHT dht(DHTPIN, DHT22);
WiFiClient espClient;
PubSubClient client(espClient);

unsigned long lastMsg = 0;
#define MSG_BUFFER_SIZE  (50)
char msg[MSG_BUFFER_SIZE];
int value = 0;
int nClient = 0;
String led_topic = "cspista/home/led";
String rgb_topic = "cspista/home/rgb";

void setup_wifi() {
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(WIFI_SSID);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("WiFi connected");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
}

void callback(char* topic, byte* payload, unsigned int length) {
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("] : ");
  for (int i = 0; i < length; i++) {
    Serial.print((char)payload[i]);
  }
  Serial.println();

  if (led_topic.equals(String(topic))) {
    // Switch on the LED if an 1 was received as first character
    if ((char)payload[0] == '1') {
      digitalWrite(BUILTIN_LED, LOW);   // LED on (pulling down cathode)
    } else {
      digitalWrite(BUILTIN_LED, HIGH);  // LED off (pulling up cathode)
    }
  }
  else if (rgb_topic.equals(String(topic))) {
    long color = strtol((char*)(payload+1), NULL, 16);
    pixels.setPixelColor(0, color);
    pixels.show();
  }
}

void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Create a random client ID
    char clientId[] = "hobbi@cspista.hu";
    // Attempt to connect
    // if (client.connect(clientId, HIVEMQ_USER, HIVEMQ_PASS)) {  // remote HiveMQ Broker    
    if (client.connect(clientId, uMQTT_USER, uMQTT_PASS)) {
      Serial.println("connected");
      // Once connected, publish an announcement...
      client.publish("cspista/home", "DHT22 connected");
      // ... and resubscribe
      client.subscribe(led_topic.c_str());
      client.subscribe(rgb_topic.c_str());   //---
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}

void setup() {
  pinMode(BUILTIN_LED, OUTPUT);     // Initialize the BUILTIN_LED pin as an output
  Serial.begin(115200);
  pixels.begin();                   // INITIALIZE NeoPixel strip object
  pixels.clear();                   // Set all pixel colors to 'off'
  pixels.setPixelColor(0, pixels.Color(0, 150, 0));
  pixels.show();                    // Send the updated pixel colors to the hardware.
  dht.begin();                      // Initialize the DHT22 sensor
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
}

void loop() {

  if (!client.connected()) {
    reconnect();
  }
  client.loop();

  unsigned long now = millis();
  if (now - lastMsg > 5000) {
    lastMsg = now;
    float t = dht.readTemperature();
    if (!isnan(t)) {
      sprintf(msg, "%5.1f", t);
      client.publish("cspista/home/temp", msg);
      Serial.print("Publish [cspista/home/temp]:");
      Serial.println(msg);
    }
    float h = dht.readHumidity();
    if (!isnan(h)) {
      sprintf(msg, "%3.0f", h);
      client.publish("cspista/home/humidity", msg);
      Serial.print("Publish [cspista/home/humidity]:");
      Serial.println(msg);
    }
  }
}
